use actix_web::{get, post, web, App, HttpResponse, HttpServer, Responder, HttpRequest, Result};
use actix_files::NamedFile;
use magick_rust::{MagickWand, ToMagick};
use magick_rust::{self, FilterType};

#[get("/status/ok")]
async fn hello() -> impl Responder {
    HttpResponse::Ok().body("{\"status\": 200}")
}

#[post("/echo")]
async fn echo(req_body: String) -> impl Responder {
    HttpResponse::Ok().body(req_body)
}

async fn manual_hello() -> impl Responder {
    HttpResponse::Ok().body("Hey there!")
}

async fn static_serve(req: HttpRequest) -> Result<NamedFile> {
    let path: &str = req.path();
    let real_path = &path[1..];

    Ok(NamedFile::open(real_path)?)
}

#[post("/image/blur")]
async fn blur_image(image_data: web::Bytes) -> Result<HttpResponse> {
    // Load the image from the incoming bytes
    let image = ToMagick::<magick_rust::MagickWand>::to_magick(image_data).unwrap();

    // Apply a blur filter to the image
    let blurred_image = image.blur(FilterType::Gaussian, 0.0).unwrap();

    // Convert the blurred image back to bytes
    let blurred_image_bytes = blurred_image.to_bytes().unwrap();

    Ok(HttpResponse::Ok()
        .content_type("image/jpeg")
        .body(blurred_image_bytes))
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    println!("Hello, world!");

    HttpServer::new(|| {
        App::new()
            .route("/static/{filename:.*}", web::get().to(static_serve))
            .service(hello)
            .service(echo)
            .service(blur_image)
            .route("/hey", web::get().to(manual_hello))
    })
    .bind(("0.0.0.0", 9090))?
    .run()
    .await
}